import org.moeaframework.Analyzer;
import org.moeaframework.Executor;
import org.moeaframework.Instrumenter;
import org.moeaframework.algorithm.NSGAII;
//import org.moeaframework.analysis.collector.Accumulator;
import org.moeaframework.core.operator.real.PM;
import org.moeaframework.core.operator.real.SBX;
import org.moeaframework.core.operator.GAVariation;
import org.moeaframework.core.operator.RandomInitialization;
import org.moeaframework.core.variable.Permutation;
import org.moeaframework.core.NondominatedPopulation;
import org.moeaframework.core.NondominatedSortingPopulation;
import org.moeaframework.core.Problem;
import org.moeaframework.core.Solution;
import org.moeaframework.core.Variation;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;


// $ javac -cp '.:jmetal-algorithm-5.5.2-jar-with-dependencies.jar:MOEAFramework-2.12-Demo.jar' -Xlint:unchecked MyMOEAFrameworkTest.java
// $ java -cp '.:jmetal-algorithm-5.5.2-jar-with-dependencies.jar:MOEAFramework-2.12-Demo.jar' MyMOEAFrameworkTest



public class MyMOEAFrameworkTest {

  public static void main(String[] args) {
  String[] arguments = {"rm", "-rf", System.getProperty("user.dir") + "/moeaframework"};
    try {
      Command.execute(arguments);
    }
    catch (InterruptedException e) {
      e.printStackTrace();
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    arguments = new String[]{"mkdir", System.getProperty("user.dir") + "/moeaframework"};
    try {
      Command.execute(arguments);
    }
    catch (InterruptedException e) {
      e.printStackTrace();
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    Problem problem = new MyMultiobjectiveTSPMOEAFramework(
                                                         "src/distance.txt", 
                                                         "src/cost.txt");
    Variation variation = new GAVariation(new SBX(15.0, 1.0), 
                                          new PM(20.0, 0.5));
    NSGAII nsgaii_test = new NSGAII(problem, 
                                    new NondominatedSortingPopulation(), 
                                    null, null, variation, 
                                    new RandomInitialization(problem, 100));
    int i = 0;
    while (i < 10) {
      nsgaii_test.step();
      i += 1;
    }
    //result = nsgaii_test.getResult();
    //for (Solution solution : result) {
    //  System.out.println(MyMultiobjectiveTSPMOEAFramework.arrayToString((Permutation)solution.getVariable(0)) + ": " + solution.getObjective(0) + "-" + solution.getObjective(1));
    //}

    String[] algorithms = {"DBEA", "eMOEA", "eNSGAII", "IBEA", "NSGAII", "NSGAIII", "Random", "RSO", "RVEA", "SPEA2", "PAES", "PESA2", "VEGA"};
    // setup the experiment
    Executor executor = new Executor()
      .withProblem(problem)
      .withMaxEvaluations(3000);
    Analyzer analyzer = new Analyzer()
      .withProblem(problem)
      .includeHypervolume()
      .showStatisticalSignificance();
    // run each algorithm for 50 seeds
    for (String algorithm : algorithms) {
      analyzer.addAll(algorithm, executor.withAlgorithm(algorithm).runSeeds(50));
    }
    // print the results
    analyzer.printAnalysis();
    /*NondominatedPopulation test_result = analyzer.getReferenceSet();
    for (Solution solution : test_result) {
      System.out.println(MyMultiobjectiveTSPMOEAFramework.arrayToString((Permutation)solution.getVariable(0)) + ": " + solution.getObjective(0) + "-" + solution.getObjective(1));
    }*/
    
    Instrumenter instrumenter = new Instrumenter()
           .withProblem(problem)
           .withFrequency(100);
    for (String algorithm : algorithms) {
      NondominatedPopulation result = new Executor()
          .withProblem(problem)
          .withAlgorithm(algorithm)   // DBEA, eMOEA, eNSGAII, IBEA, NSGAII, NSGAIII, Random, RSO, RVEA, SPEA2, PAES, PESA2, VEGA, !GDE3, !MSOPS
          .withMaxEvaluations(30000)
          .withProperty("populationSize", 100)
          .withProperty("sbx.rate", 1.0)
          .withProperty("sbx.distributionIndex", 15.0)
          .withProperty("pm.rate", 0.05)
          .withProperty("pm.distributionIndex", 20.0)
          //.withCheckpointFrequency(1000)
          //.withCheckpointFile(new File("example.txt"))
          //.distributeOnAllCores()
          .withInstrumenter(instrumenter)
          .run();
      //Analyzer analyzer = new Analyzer()
      //          .withProblem(problem)
      //          .includeHypervolume()
      //          .showStatisticalSignificance();
      // print the results
      //System.out.println(analyzer.printAnalysis());
      //Accumulator accumulator = instrumenter.getLastAccumulator();
      //System.out.println(accumulator.toCSV());
      PrintWriter writer, writer_1;
      try {
        writer = new PrintWriter("moeaframework/" + algorithm + "_nondominated_solutions.txt", "UTF-8");
        writer_1 = new PrintWriter("moeaframework/" + algorithm + "_nondominated_fitness.txt", "UTF-8");
        // display the results
        for (Solution solution : result) {
          //System.out.println(MyMultiobjectiveTSPMOEAFramework.arrayToString((Permutation)solution.getVariable(0)) + ": " + solution.getObjective(0) + "-" + solution.getObjective(1));
          writer.println(MyMultiobjectiveTSPMOEAFramework.arrayToString((Permutation)solution.getVariable(0)));
          writer_1.println(solution.getObjective(0) + " " + solution.getObjective(1));
        }
        writer.close();
        writer_1.close();
      }
      catch (FileNotFoundException e) {
        e.printStackTrace();
      }
      catch (UnsupportedEncodingException e) {
        e.printStackTrace();
      }
    }
  }
}
