import org.moeaframework.core.Solution;
import org.moeaframework.core.Variable;
import org.moeaframework.core.variable.Permutation;
import org.moeaframework.problem.AbstractProblem;



public class MyMultiobjectiveTSPMOEAFramework extends AbstractProblem {

  protected int     numberOfCities;
  public double[][] distanceMatrix;
  public double[][] costMatrix;

  public MyMultiobjectiveTSPMOEAFramework(String distanceFile, 
                                          String costFile) {
    super(1, 2);
    this.distanceMatrix = ReadFile.readData(distanceFile);
    this.costMatrix     = ReadFile.readData(costFile);
    this.numberOfCities = distanceMatrix.length;
    //System.out.println(getNumberOfVariables());
  }


  public Solution newSolution() {
    Solution solution = new Solution(getNumberOfVariables(), 
                                     getNumberOfObjectives());

    for (int i = 0; i < getNumberOfVariables(); i++) {
      solution.setVariable(i, new Permutation(this.numberOfCities));
    }

    return solution;
  }


  public void evaluate(Solution solution) {
    double fitness1;
    double fitness2;

    fitness1 = 0.0;
    fitness2 = 0.0;
    
    Permutation aux = (Permutation)solution.getVariable(0);
    //solution.getVariable() returns a Variable object

    int i;
    int ini = aux.get(0);
    for (i = 0; i < (this.numberOfCities - 1); i++) {
      int x;
      int y;

      x = aux.get(i);
      y = aux.get(i + 1);

      fitness1 += this.distanceMatrix[x][y];
      fitness2 += this.costMatrix[x][y];
    }
    int fin = aux.get(i);
    fitness1 += this.distanceMatrix[fin][ini];
    fitness2 += this.costMatrix[fin][ini];
    //System.out.println(arrayToString(aux) + ": " + ini + "-" + fin);
    //System.out.println(arrayToString(aux) + ": " + fitness1 + "-" + 
    //                   fitness2);

    solution.setObjective(0, fitness1);
    solution.setObjective(1, fitness2);
  }


  public static String arrayToString(Permutation p) {
    String res = "";
    int[] aux = p.toArray();
    for (int i = 0; i < aux.length; i++) {
      if (i != aux.length - 1) {
        res += aux[i] + " ";
      }
      else {
        res += aux[i];
      }
    }
    return res;

    /*String res = "[";
    int[] aux = p.toArray();
    for (int i = 0; i < aux.length; i++) {
      if (i != aux.length - 1) {
        res += aux[i] + ", ";
      }
      else {
        res += aux[i] + "]";
      }
    }
    return res;*/
  }


  public static void main(String[] args) {
    Permutation p = new Permutation(17);
    p.randomize();
    int[] a = p.toArray();
    for (int i = 0; i < a.length; i++) {
      System.out.println(a[i]);
    }
    Variable v = new Permutation(17);
    v.randomize();
    Solution s = new Solution(1, 2);
    s.setVariable(0, v);
  }
}
