import org.uma.jmetal.algorithm.Algorithm;
import org.uma.jmetal.algorithm.multiobjective.espea.ESPEA;
import org.uma.jmetal.algorithm.multiobjective.gwasfga.GWASFGA;
import org.uma.jmetal.algorithm.multiobjective.ibea.IBEA;
import org.uma.jmetal.algorithm.multiobjective.nsgaii.NSGAII;
import org.uma.jmetal.algorithm.multiobjective.nsgaii.NSGAIIBuilder;
import org.uma.jmetal.algorithm.multiobjective.nsgaiii.NSGAIII;
import org.uma.jmetal.algorithm.multiobjective.nsgaiii.NSGAIIIBuilder;
import org.uma.jmetal.algorithm.multiobjective.paes.PAES;
import org.uma.jmetal.algorithm.multiobjective.paes.PAESBuilder;
//import org.uma.jmetal.algorithm.multiobjective.pesa2.PESA2;
//import org.uma.jmetal.algorithm.multiobjective.pesa2.PESA2Builder;
import org.uma.jmetal.algorithm.multiobjective.smsemoa.SMSEMOA;
import org.uma.jmetal.algorithm.multiobjective.spea2.SPEA2;
import org.uma.jmetal.algorithm.multiobjective.spea2.SPEA2Builder;
import org.uma.jmetal.algorithm.multiobjective.wasfga.WASFGA;
import org.uma.jmetal.operator.CrossoverOperator;
import org.uma.jmetal.operator.MutationOperator;
import org.uma.jmetal.operator.SelectionOperator;
//import org.uma.jmetal.operator.impl.crossover.NPointCrossover;
import org.uma.jmetal.operator.impl.crossover.PMXCrossover;
//import org.uma.jmetal.operator.impl.mutation.NullMutation;
import org.uma.jmetal.operator.impl.mutation.PermutationSwapMutation;
//import org.uma.jmetal.operator.impl.mutation.SimpleRandomMutation;
import org.uma.jmetal.operator.impl.selection.BinaryTournamentSelection;
//import org.uma.jmetal.operator.impl.selection.TournamentSelection;
//import org.uma.jmetal.problem.multiobjective.MultiobjectiveTSP;
import org.uma.jmetal.problem.PermutationProblem;
//import org.uma.jmetal.solution.DoubleSolution;
import org.uma.jmetal.solution.PermutationSolution;
import org.uma.jmetal.util.AlgorithmRunner;
import org.uma.jmetal.util.AbstractAlgorithmRunner;
import org.uma.jmetal.util.JMetalLogger;
import org.uma.jmetal.util.comparator.RankingAndCrowdingDistanceComparator;
import org.uma.jmetal.util.evaluator.SolutionListEvaluator;
import org.uma.jmetal.util.evaluator.impl.SequentialSolutionListEvaluator;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;


// $ javac -cp '.:jmetal-algorithm-5.5.2-jar-with-dependencies.jar:MOEAFramework-2.12-Demo.jar' -Xlint:unchecked MyjMetalTest.java
// $ java -cp '.:jmetal-algorithm-5.5.2-jar-with-dependencies.jar:MOEAFramework-2.12-Demo.jar' MyjMetalTest



public class MyjMetalTest extends AbstractAlgorithmRunner {

  public static void generateFilePopulation(String algorithm, List<PermutationSolution<Integer>> population, boolean nondominated) {
    PrintWriter writer, writer_1;
    String fitness, solution;
    try {
      if (nondominated) {
        writer = new PrintWriter("jMetal/" + algorithm + "_nondominated_solutions.txt", "UTF-8");
        writer_1 = new PrintWriter("jMetal/" + algorithm + "_nondominated_fitness.txt", "UTF-8");
      }
      else {
        writer = new PrintWriter("jMetal/" + algorithm + "_solutions.txt", "UTF-8");
        writer_1 = new PrintWriter("jMetal/" + algorithm + "_fitness.txt", "UTF-8");
      }
      for (int i = 0; i < population.size(); i++) {
        solution = "";
        for (int j = 0; j < population.get(i).getNumberOfVariables(); j++) {
          if (j != population.get(i).getNumberOfVariables())
            solution += population.get(i).getVariableValue(j) + " ";
          else
            solution += population.get(i).getVariableValue(j);
        }
        fitness = population.get(i).getObjective(0) + " " + population.get(i).getObjective(1);
        writer.println(solution);
        writer_1.println(fitness);
      }
      writer.close();
      writer_1.close();
    }
    catch (FileNotFoundException e) {
      e.printStackTrace();
    }
    catch (UnsupportedEncodingException e) {
      e.printStackTrace();
    }
  }



  public static void main(String[] argv) throws Exception {
    String[] arguments = {"rm", "-rf", System.getProperty("user.dir") + "/jMetal"};
    try {
      Command.execute(arguments);
    }
    catch (InterruptedException e) {
      e.printStackTrace();
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    arguments = new String[]{"mkdir", System.getProperty("user.dir") + "/jMetal"};
    try {
      Command.execute(arguments);
    }
    catch (InterruptedException e) {
      e.printStackTrace();
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    try {
      PermutationProblem<PermutationSolution<Integer>> problem;
      CrossoverOperator<PermutationSolution<Integer>> crossover;
      MutationOperator<PermutationSolution<Integer>> mutation;
      SelectionOperator<List<PermutationSolution<Integer>>, PermutationSolution<Integer>> selection;
      SolutionListEvaluator<PermutationSolution<Integer>> evaluator;
      problem = new MyMultiobjectiveTSPjMetal("src/distance.txt", 
                                              "src/cost.txt");
      //problem.getNumberOfVariables();
      crossover = new PMXCrossover(0.9);
      mutation = new PermutationSwapMutation<Integer>(0.5);
      //mutation = new NullMutation<PermutationSolution<Integer>>();
      selection = new BinaryTournamentSelection<PermutationSolution<Integer>>(new RankingAndCrowdingDistanceComparator<PermutationSolution<Integer>>());
      evaluator = new SequentialSolutionListEvaluator<PermutationSolution<Integer>>();

      Algorithm<List<PermutationSolution<Integer>>> algorithm;
      AlgorithmRunner algorithmRunner;
      List<PermutationSolution<Integer>> population;
      String nameAlgorithm = "";
      String referenceParetoFront = "";
      long computingTime;

      int maxIterations = 10000;
      int populationSize = 100;
      int archiveSize = 50;
      int biSections = 8;

      nameAlgorithm = "ESPEA";
      ESPEA<PermutationSolution<Integer>> espea_test = new ESPEA<PermutationSolution<Integer>>(problem, maxIterations, populationSize, crossover, crossover, mutation, selection, null, evaluator, false, null);
      //System.out.println(espea_test.getResult());   // null
      //System.out.println(espea_test.getPopulation());   // null
      espea_test.run();
      System.out.println(nameAlgorithm);
      //System.out.println(espea_test.getPopulation() + "\n");
      System.out.println(espea_test.getResult() + "\n\n");
      MyjMetalTest.generateFilePopulation(nameAlgorithm, espea_test.getPopulation(), false);
      MyjMetalTest.generateFilePopulation(nameAlgorithm, espea_test.getResult(), true);
      //System.exit(0);

      nameAlgorithm = "GWASFGA";
      GWASFGA<PermutationSolution<Integer>> gwasfga_test = new GWASFGA<PermutationSolution<Integer>>(problem, populationSize, 100, crossover, mutation, selection, evaluator, 100);
      //System.out.println(gwasfga_test.getResult());   // null
      //System.out.println(gwasfga_test.getPopulation());   // null
      gwasfga_test.run();
      System.out.println(nameAlgorithm);
      //System.out.println(gwasfga_test.getPopulation() + "\n");
      System.out.println(gwasfga_test.getResult() + "\n\n");
      MyjMetalTest.generateFilePopulation(nameAlgorithm, gwasfga_test.getPopulation(), false);
      MyjMetalTest.generateFilePopulation(nameAlgorithm, gwasfga_test.getResult(), true);

      //maxIterations = 10000;
      nameAlgorithm = "IBEA";
      IBEA<PermutationSolution<Integer>> ibea_test = new IBEA<PermutationSolution<Integer>>(problem, populationSize, archiveSize, maxIterations, selection, crossover, mutation);
      //System.out.println(ibea_test.getResult());   // null
      ibea_test.run();
      System.out.println(nameAlgorithm);
      System.out.println(ibea_test.getResult() + "\n\n");
      MyjMetalTest.generateFilePopulation(nameAlgorithm, ibea_test.getResult(), true);

      //maxIterations = 10;
      nameAlgorithm = "NSGAII";
      NSGAII<PermutationSolution<Integer>> nsgaii_test = new NSGAII<PermutationSolution<Integer>>(problem, maxIterations, 
                                      populationSize, crossover, mutation, 
                                      selection, evaluator);
      //System.out.println(nsgaii_test.getResult());   // null
      //System.out.println(nsgaii_test.getPopulation());   // null
      nsgaii_test.run();
      System.out.println(nameAlgorithm);
      //System.out.println(nsgaii_test.getPopulation() + "\n");
      System.out.println(nsgaii_test.getResult() + "\n\n");
      MyjMetalTest.generateFilePopulation(nameAlgorithm, nsgaii_test.getPopulation(), false);
      MyjMetalTest.generateFilePopulation(nameAlgorithm, nsgaii_test.getResult(), true);

      //maxIterations = 50000;
      nameAlgorithm = "NSGAIII";
      NSGAIIIBuilder<PermutationSolution<Integer>> nsgaiii_builder = new NSGAIIIBuilder<PermutationSolution<Integer>>(problem)
              .setSelectionOperator(selection)
              .setCrossoverOperator(crossover)
              .setMutationOperator(mutation)
              .setMaxIterations(maxIterations);
      NSGAIII<PermutationSolution<Integer>> nsgaiii_test = new NSGAIII<PermutationSolution<Integer>>(nsgaiii_builder);
      //or nsgaiii_test = nsgaiii_builder.build();
      nsgaiii_test.run();
      System.out.println(nameAlgorithm);
      //System.out.println(nsgaiii_test.getPopulation() + "\n");
      System.out.println(nsgaiii_test.getResult() + "\n\n");
      MyjMetalTest.generateFilePopulation(nameAlgorithm, nsgaiii_test.getPopulation(), false);
      MyjMetalTest.generateFilePopulation(nameAlgorithm, nsgaiii_test.getResult(), true);

      nameAlgorithm = "PAES";
      PAES<PermutationSolution<Integer>> paes_test = new PAES<PermutationSolution<Integer>>(problem, archiveSize, maxIterations, 
                                      biSections, mutation);
      paes_test.run();
      System.out.println(nameAlgorithm);
      //System.out.println(paes_test.getPopulation() + "\n");
      System.out.println(paes_test.getResult() + "\n\n");
      MyjMetalTest.generateFilePopulation(nameAlgorithm, paes_test.getPopulation(), false);
      MyjMetalTest.generateFilePopulation(nameAlgorithm, paes_test.getResult(), true);

      nameAlgorithm = "SMSEMOA";
      SMSEMOA<PermutationSolution<Integer>> smsemoa_test = new SMSEMOA<PermutationSolution<Integer>>(problem, 1500, 2000, 2.0, crossover, mutation, selection, null, null);
      //System.out.println(smsemoa_test.getResult());   // null
      //System.out.println(smsemoa_test.getPopulation());   // null
      smsemoa_test.run();
      System.out.println(nameAlgorithm);
      //System.out.println(smsemoa_test.getPopulation() + "\n");
      System.out.println(smsemoa_test.getResult() + "\n\n");
      MyjMetalTest.generateFilePopulation(nameAlgorithm, smsemoa_test.getPopulation(), false);
      MyjMetalTest.generateFilePopulation(nameAlgorithm, smsemoa_test.getResult(), true);

      //maxIterations = 100;
      nameAlgorithm = "SPEA2";
      SPEA2<PermutationSolution<Integer>> spea2_test = new SPEA2<PermutationSolution<Integer>>(problem, maxIterations, 
                                      populationSize, crossover, mutation, 
                                      selection, evaluator);
      spea2_test.run();
      System.out.println(nameAlgorithm);
      //System.out.println(spea2_test.getPopulation() + "\n");
      System.out.println(spea2_test.getResult() + "\n\n");
      MyjMetalTest.generateFilePopulation(nameAlgorithm, spea2_test.getPopulation(), false);
      MyjMetalTest.generateFilePopulation(nameAlgorithm, spea2_test.getResult(), true);

      List<Double> referencePoint = new ArrayList<>();
      for (int i = 0; i < problem.getNumberOfObjectives(); i++) {
        referencePoint.add(0.0);   //initialization
      }
      nameAlgorithm = "WASFGA";
      WASFGA<PermutationSolution<Integer>> wasfga_test = new WASFGA<PermutationSolution<Integer>>(problem, populationSize, maxIterations, crossover, mutation, selection, evaluator, 1000, referencePoint);
      //System.out.println(wasfga_test.getResult());   // null
      //System.out.println(wasfga_test.getPopulation());   // null
      wasfga_test.run();
      System.out.println(nameAlgorithm);
      //System.out.println(wasfga_test.getPopulation() + "\n");
      System.out.println(wasfga_test.getResult() + "\n\n");
      MyjMetalTest.generateFilePopulation(nameAlgorithm, wasfga_test.getPopulation(), false);
      MyjMetalTest.generateFilePopulation(nameAlgorithm, wasfga_test.getResult(), true);
      System.exit(0);

      algorithm = new NSGAIIBuilder<PermutationSolution<Integer>>(problem, 
                                                                  crossover, 
                                                                  mutation)
                              .setSelectionOperator(selection)
                              .setMaxEvaluations(maxIterations)
                              .setPopulationSize(populationSize)
                              .build();
      algorithmRunner = new AlgorithmRunner.Executor(algorithm).execute();
      population = algorithm.getResult();
      computingTime = algorithmRunner.getComputingTime();
      System.out.println("NSGAIIBuilder");
      JMetalLogger.logger.info("Total execution time: " + computingTime + 
                               "ms");
      AbstractAlgorithmRunner.printFinalSolutionSet(population);
      if (!referenceParetoFront.equals("")) {
        AbstractAlgorithmRunner.printQualityIndicators(population, 
        referenceParetoFront);
      }
      System.out.println("\n");

      //maxIterations = 100;
      referenceParetoFront = "";
      algorithm = new SPEA2Builder<PermutationSolution<Integer>>(problem, 
                                                                 crossover,
                                                                 mutation)
                              .setSelectionOperator(selection)
                              .setMaxIterations(maxIterations)
                              .setPopulationSize(populationSize)
                              .build();
      algorithmRunner = new AlgorithmRunner.Executor(algorithm).execute();
      population = algorithm.getResult();
      computingTime = algorithmRunner.getComputingTime();
      System.out.println("SPEA2Builder");
      JMetalLogger.logger.info("Total execution time: " + computingTime + 
                               "ms");
      AbstractAlgorithmRunner.printFinalSolutionSet(population);
      if (!referenceParetoFront.equals("")) {
        AbstractAlgorithmRunner.printQualityIndicators(population, 
                                                       referenceParetoFront);
      }
      System.out.println("\n");

      referenceParetoFront = "";
      algorithm = new PAESBuilder<PermutationSolution<Integer>>(problem)
                               .setArchiveSize(archiveSize)
                               .setMutationOperator(mutation)
                               .setBiSections(biSections)
                               .setMaxEvaluations(maxIterations)
                               .build();
      algorithmRunner = new AlgorithmRunner.Executor(algorithm).execute();
      population = algorithm.getResult();
      computingTime = algorithmRunner.getComputingTime();
      System.out.println("PAESBuilder");
      JMetalLogger.logger.info("Total execution time: " + computingTime + 
                               "ms");
      AbstractAlgorithmRunner.printFinalSolutionSet(population);
      if (!referenceParetoFront.equals("")) {
        AbstractAlgorithmRunner.printQualityIndicators(population, 
                                                       referenceParetoFront);
      }
      System.out.println("\n");

      // NOT SHOW ANYTHING!!!
      /*PESA2<PermutationSolution<Integer>> pesa2_test = new PESA2<PermutationSolution<Integer>>(problem, 100, 1000, 
                                    100, 12, crossover, 
                                    mutation, evaluator);
      pesa2_test.run();
      System.out.println("PESA2");
      //System.out.println(pesa2_test.getPopulation() + "\n");
      System.out.println(pesa2_test.getResult() + "\n\n");
      MyjMetalTest.generateFilePopulation(nameAlgorithm, pesa2_test.getPopulation(), false);
      MyjMetalTest.generateFilePopulation(nameAlgorithm, pesa2_test.getResult(), true);
      
      referenceParetoFront = "";
      algorithm = new PESA2Builder<PermutationSolution<Integer>>(problem, 
                                                                 crossover, 
                                                                 mutation)
                               .setArchiveSize(archiveSize)
                               .setBisections(biSections)
                               .setMaxEvaluations(maxIterations)
                               .build();
      algorithmRunner = new AlgorithmRunner.Executor(algorithm).execute();
      population = algorithm.getResult();
      computingTime = algorithmRunner.getComputingTime();
      System.out.println("PESA2Builder");
      JMetalLogger.logger.info("Total execution time: " + computingTime + 
                               "ms");
      AbstractAlgorithmRunner.printFinalSolutionSet(population);
      if (!referenceParetoFront.equals("")) {
        AbstractAlgorithmRunner.printQualityIndicators(population, referenceParetoFront);
      }
      System.out.println("\n");*/
    }
    catch (IOException e) {
      System.out.println("ERROR");
    }
  }
}
