#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import geopy.distance
import googlemaps
import math
import matplotlib.pyplot as plt
import pickle
import random
import subprocess
from datetime import datetime
from geopy.geocoders import Nominatim
from mpl_toolkits.basemap import Basemap


CITIES = {0: "Gerona, España", 1: "Barcelona, España", \
          2: "Tarragona, España", 3: "Tortosa, España", \
          4: "Castellon, España", 5: "Valencia, España", \
          6: "Alicante, España", 7: "Murcia, España", \
          8: "Almeria, España", 9: "Malaga, España", \
          10: "Marbella, España", 11: "Gibraltar, Gibraltar", \
          12: "Cadiz, España", 13: "Huelva, España", \
          14: "Badajoz, España", 15: "Caceres, España", \
          16: "Salamanca, España"}



def distance_cities(city_1="Gerona, España", city_2="Barcelona, España"):
  """
  Method to calculate distance between two cities with geopy library, 
  that takes info from Google
  """
  geolocator = Nominatim()
  location_1 = geolocator.geocode(city_1)
  location_2 = geolocator.geocode(city_2)

  return geopy.distance.vincenty((location_1.latitude, \
                                  location_1.longitude), 
                                 (location_2.latitude, \
                                  location_2.longitude)).km



def google_info(city_1="Gerona, España", city_2="Barcelona, España"):
  gmaps = googlemaps.Client(key="AIzaSyCHTcD4vxy5WgArj2IC8STpOOK7x9Qn-_E")

  #for mode_transport in ["driving", "bicycling", "walking", "transit"]:
  for mode_transport in ["driving"]:
    now = datetime.now()
    #print(mode_transport)
    result = gmaps.distance_matrix([city_1], [city_2], \
                                   departure_time=now, \
                                   mode=mode_transport, \
                                   language="es-ES")
    return (result["rows"][0]["elements"][0]["distance"]["value"], \
    #        random.randint(0, 100))
            result["rows"][0]["elements"][0]["duration"]["value"])
    #print(result["rows"][0]["elements"][0]["distance"]["value"])
    #print(result["rows"][0]["elements"][0]["duration"]["value"])
    #print()
    #if mode_transport == "transit":
      #for mode_transit in ["bus", "subway", "train", "tram", "rail"]:
        #print(mode_transport + "-" + mode_transit)
        #result = gmaps.distance_matrix([city_1], \
        #                               [city_2], departure_time=now, \
        #                               mode=mode_transport, \
        #                               language="es-ES", \
        #                               transit_mode=mode_transit)
        #print(result)
        #print()
    #else:
      #print(mode_transport)
      #result = gmaps.distance_matrix([city_1], [city_2], \
      #                               departure_time=now, \
      #                               mode=mode_transport, \
      #                               language="es-ES")
      #print(result)
      #print()



def tour_eeuu():
  m = Basemap(projection="merc", llcrnrlat=20, urcrnrlat=50, \
              llcrnrlon=-130, urcrnrlon=-60, resolution="i", \
              area_thresh=10000.)
  m.drawcoastlines(linewidth=0.5)
  m.drawcountries(linewidth=0.5)
  #m.drawstates()
  m.fillcontinents(color="#04BAE3", lake_color="#FFFFFF")
  m.drawmapboundary(fill_color="#FFFFFF")

  cities = ["Nueva York", "Philadelphia", "Charlotte", "Jacksonville", \
            "Atlanta", "Nashville", "Dallas", "Albuquerque, New Mexico", \
            "Las Vegas", "Los Angeles", "Sacramento", "Seattle"]

  geolocator = Nominatim()
  coor_x = []
  coor_y = []
  for name in cities:
    location = geolocator.geocode(name)
    x, y = m(location.longitude, location.latitude)
    coor_x.append(x)
    coor_y.append(y)
    m.plot(x, y, "ro")

  m.plot(coor_x, coor_y, "g-")

  plt.title("EEUU TOUR")
  plt.show()



def tour_spain():
  m = Basemap(projection="merc", llcrnrlat=35, urcrnrlat=45, \
              llcrnrlon=-10, urcrnrlon=5, resolution="i", \
              area_thresh=500.)
  m.drawcoastlines(linewidth=0.5)
  m.drawcountries(linewidth=0.5)
  m.drawstates()
  m.fillcontinents(color="#04BAE3", lake_color="#FFFFFF")
  m.drawmapboundary(fill_color="#FFFFFF")

  geolocator = Nominatim()
  coor_x = []
  coor_y = []
  for num in CITIES:
    location = geolocator.geocode(CITIES[num])
    x, y = m(location.longitude, location.latitude)
    coor_x.append(x)
    coor_y.append(y)
    m.plot(x, y, "ro")

  m.plot(coor_x, coor_y, "g-")

  plt.title("SPAIN TOUR")
  plt.show()




if __name__ == "__main__":
  subprocess.check_output("rm -rf ../*.txt", shell=True)

  aux = sorted(CITIES.items())
  #print(aux)
  #print(google_info(city_2="Tarragona, España"))
  #print(distance_cities())



  #tour_spain()



  # googlemaps
  """
  matrix = []
  for x in range(len(aux)):
    matrix.append([None] * len(aux))

  i = 0
  for c1 in aux:
    j = i
    while j < len(aux):
      print("{}-{}".format(c1[1], aux[j][1]))
      if c1[1] == aux[j][1]:   # same city
        pass
      else:
        matrix[i][j] = (google_info(c1[1], aux[j][1]))
        #matrix[i][j] = (distance_cities(c1[1], aux[j][1]))
      j += 1
    i += 1
    print()
  
  #print(matrix)
  """

  matrix = [[None, (104174, 4916), (191452, 7203), (271732, 9872), (373070, 13537), (442382, 15307), (629755, 21392), (669435, 23008), (878774, 30318), (1088161, 36721), (1142658, 37978), (1217621, 41118), (1208140, 40524), (1181033, 39828), (1099614, 36482), (996721, 33295), (921813, 30732)], [None, None, (100014, 4552), (180294, 7221), (281632, 10886), (350944, 12655), (538317, 18741), (577998, 20357), (787336, 27667), (996723, 34070), (1051220, 35327), (1126183, 38467), (1116702, 37873), (1089596, 37176), (997824, 35743), (970436, 32769), (798639, 28760)], [None, None, None, (87943, 3474), (189282, 7139), (258593, 8908), (445967, 14994), (485647, 16610), (694985, 23920), (875909, 30231), (930405, 31488), (1005368, 34628), (1024352, 34126), (997245, 33429), (905474, 31996), (878085, 29022), (722019, 26058)], [None, None, None, None, (126521, 5415), (195833, 7184), (383206, 13270), (422886, 14886), (632225, 22196), (813148, 28507), (867645, 29764), (942608, 32904), (961591, 32402), (934484, 31705), (842713, 30272), (815324, 27298), (759305, 25742)], [None, None, None, None, None, (108284, 4837), (295657, 10923), (335337, 12539), (544676, 19849), (725599, 26160), (780096, 27417), (855059, 30557), (874042, 30055), (846935, 29358), (755164, 27925), (727775, 24951), (671756, 23395)], [None, None, None, None, None, None, (169495, 6809), (218252, 8450), (427590, 15761), (608513, 22072), (663010, 23329), (737973, 26468), (774990, 26515), (747883, 25818), (656112, 24385), (628723, 21411), (572703, 19855)], [None, None, None, None, None, None, None, (81456, 3414), (290795, 10724), (471718, 17035), (526215, 18293), (601177, 21432), (640167, 23612), (691611, 24171), (687901, 25368), (672273, 22314), (636865, 21827)], [None, None, None, None, None, None, None, None, (218345, 7967), (399269, 14278), (453766, 15535), (528728, 18675), (567718, 20854), (619162, 21414), (666296, 24252), (650667, 21198), (615259, 20710)], [None, None, None, None, None, None, None, None, None, (201079, 7668), (261569, 9581), (336532, 12721), (436949, 16136), (508833, 17789), (621946, 22330), (675597, 23277), (733205, 26293)], [None, None, None, None, None, None, None, None, None, None, (60731, 3034), (135693, 6173), (236111, 9588), (301937, 11586), (419596, 16749), (419733, 17831), (647151, 23916)], [None, None, None, None, None, None, None, None, None, None, None, (78205, 3789), (178623, 7204), (281583, 12970), (463350, 16986), (517002, 17933), (714347, 24032)], [None, None, None, None, None, None, None, None, None, None, None, None, (120325, 5775), (286529, 10843), (405053, 15557), (458704, 16504), (656050, 22603)], [None, None, None, None, None, None, None, None, None, None, None, None, None, (210104, 7614), (328627, 12329), (382279, 13275), (579625, 19374)], [None, None, None, None, None, None, None, None, None, None, None, None, None, None, (237132, 11572), (347414, 12154), (544760, 18252)], [None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, (92611, 4675), (291812, 10798)], [None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, (201285, 7193)], [None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None]]

  print(matrix)

  i = 0
  while i < len(matrix):
    j = i
    while j < len(aux):
      if matrix[i][j] != None:
        #print("{}-{}".format(i, j))
        #matrix[i][j] = (math.floor(matrix[i][j][0] / 1000), \
        #                math.floor(random.uniform(0.1, 1) * \
        #                           matrix[i][j][0] / 1000))   # good_data
        matrix[i][j] = (matrix[i][j][0], matrix[i][j][1])   # bad data
      j += 1
    i += 1

  print(matrix)

  # fill the other diagonal part of matrix
  i = 0
  while i < len(matrix):
    j = 0
    while j < i:
      #print("{}-{}".format(i, j))
      matrix[i][j] = matrix[j][i]
      j += 1
    i += 1

  print(matrix)

  i = 0
  while i < len(matrix):
    matrix[i][i] = (999999999, 999999999)
    i += 1

  print(matrix)

  # cut some cities
  #i = 0
  #matrix = matrix[:9]
  #while i < len(matrix):
  #  matrix[i] = matrix[i][:9]
  #  i += 1



  # geopy
  """
  # random cost (peaje)
  matrix = [[None, 85.14562641055582, 161.93381455444515, 231.93952332174376, 308.79025007141826, 387.95679526023065, 491.411101950813, 556.4701124424378, 729.3176753342007, 854.0312787446766, 900.0300983339217, 958.6663995223872, 991.3425117124087, 988.1706011697022, 898.925219742867, 824.3141418494105, 717.2644047832102], [None, None, 82.78334188875209, 152.96884646240906, 226.8270846555153, 303.40982385209065, 406.41130817900205, 471.3289069848971, 644.1912914049046, 770.3567914513407, 816.5974419652074, 875.2374335516138, 909.9091394205666, 909.8082635375122, 827.5810846090924, 755.3980516656819, 659.4099701388935], [None, None, None, 70.5042200550679, 147.01075546155914, 229.51394666363356, 341.32265124139843, 402.9343034704047, 573.5205601348715, 692.8596045757863, 738.6195367311115, 797.2373833166172, 829.4453408166204, 827.64057890484, 745.0141518169839, 673.3259733751062, 581.829237409012], [None, None, None, None, 79.31350588697694, 167.38030815125438, 286.31245530293523, 343.7289149483906, 511.1187201102868, 624.7946360350735, 670.186027882614, 728.7501408231353, 759.488017792549, 757.1364414528596, 676.147684808457, 605.6329491749236, 521.4232574907683], [None, None, None, None, None, 90.89539924737274, 213.95013120047872, 267.3293656568198, 432.55813297088713, 546.0330416219895, 591.6681808408873, 650.2754765940741, 683.1021041957144, 684.4411579910138, 613.0829461756239, 546.6552579361659, 480.62851956300875], [None, None, None, None, None, None, 124.30500067332294, 176.65544652061118, 344.3260515387513, 467.90109948729, 514.6055439930201, 573.1759337684714, 613.4299395715311, 624.5053705731541, 573.7672726511771, 515.823170966485, 479.67622678669863], [None, None, None, None, None, None, None, 68.9736471094517, 241.8978918526002, 391.8149304040502, 439.5400360293876, 496.4287514551325, 551.8064663069063, 581.4760050211378, 567.4968028313477, 524.9533178211739, 530.134949456799], [None, None, None, None, None, None, None, None, 173.85363234595246, 323.87672934022925, 371.5612555784858, 428.0869129000796, 485.7369404038001, 519.9399814900945, 519.4075444946257, 484.41736639347516, 510.8265324063306], [None, None, None, None, None, None, None, None, None, 175.36029067608754, 219.68509286488893, 270.4666716327514, 344.0094433904543, 401.643905667204, 456.63386655564483, 450.23650719803, 535.3281144602049], [None, None, None, None, None, None, None, None, None, None, 47.728430798598716, 105.55097568046527, 168.68843922555268, 232.6822875376382, 328.2164665382233, 350.1905156780144, 483.2941615329357], [None, None, None, None, None, None, None, None, None, None, None, 58.64117565107028, 126.03867633317446, 201.788351440815, 320.8744555940873, 354.0880818554955, 499.28468398708173], [None, None, None, None, None, None, None, None, None, None, None, None, 94.66774675887451, 188.82715339222437, 335.7052373223806, 380.70514183918294, 536.2168743977783], [None, None, None, None, None, None, None, None, None, None, None, None, None, 99.70408161964635, 267.3247591954391, 326.9241233040452, 495.38758854066424], [None, None, None, None, None, None, None, None, None, None, None, None, None, None, 179.74904271896438, 251.19524669675175, 426.3195642704184], [None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, 84.29374736259132, 257.46655690776373], [None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, 176.1367722293517], [None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None]]

  i = 0
  while i < len(matrix):
    j = i
    while j < len(aux):
      if i == j:
        matrix[i][j] = (999999999, 999999999)
      elif matrix[i][j] != None:
        # asign random cost
        matrix[i][j] = (math.floor(matrix[i][j]), \
                        math.floor(random.uniform(0.1, 1) * \
                                   math.floor(matrix[i][j])))
      j += 1
    i += 1
  
  print(matrix)
  
  # fill the other diagonal part of matrix
  i = 0
  while i < len(matrix):
    j = 0
    while j < i:
      #print("{}-{}".format(i, j))
      matrix[i][j] = matrix[j][i]
      j += 1
    i += 1

  print(matrix)
  """



  # save data
  with open("../distance.txt", "w") as f:
    i = 0
    while i < len(matrix):
      j = 0
      while j < len(matrix[i]):
        if j == len(matrix[i]) - 1:
          f.write(str(matrix[i][j][0]) + "\n")
        else:
          f.write(str(matrix[i][j][0]) + " ")
        j += 1
      i += 1
  with open("../cost.txt", "w") as f:
    i = 0
    while i < len(matrix):
      j = 0
      while j < len(matrix[i]):
        if j == len(matrix[i]) - 1:
          f.write(str(matrix[i][j][1]) + "\n")
        else:
          f.write(str(matrix[i][j][1]) + " ")
        j += 1
      i += 1

  with open("../tsp_python.txt", "wb") as f:
    pickle.dump([matrix, CITIES], f, protocol=2)
  
  subprocess.check_output("chmod 755 ../tsp_python.txt", shell=True)
  subprocess.check_output("chmod 755 ../distance.txt", shell=True)
  subprocess.check_output("chmod 755 ../cost.txt", shell=True)
