#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# plot the results using matplotlib
import matplotlib.pyplot as plt
import matplotlib.ticker as ticker
import os




if __name__ == "__main__":
  algorithms = ["ESPEA", "GWASFGA", "IBEA", "NSGAII", "NSGAIII", \
                "SMSEMOA", "SPEA2", "PAES", "WASFGA"]
  # display the results
  fig = plt.figure()
  i = 1
  for name_algorithm in algorithms:
    ax = fig.add_subplot(3, 3, i)
    if os.path.exists(os.getcwd() + \
                      "jMetal/{}_fitness.txt".format(name_algorithm)):
      fitness_solutions = []
      with open("jMetal/{}_fitness.txt".format(name_algorithm), "r") as f:
        for line in f.readlines():
          fitness_solutions.append((float(line.split()[0]), \
                                    float(line.split()[1])))
      ax.scatter([s[0] for s in fitness_solutions], \
                 [s[1] for s in fitness_solutions], s=12, c="g", marker=".")
    nondominated_solutions = []
    with open("jMetal/{}_nondominated_fitness.txt".format(name_algorithm), \
              "r") as f:
      for line in f.readlines():
        nondominated_solutions.append((float(line.split()[0]), \
                                       float(line.split()[1])))
    l = list((s[0], s[1]) for s in nondominated_solutions)
    l = sorted(l, key=lambda x : (x[0], x[1]))
    j = 0
    while j < len(l) - 1:
      ax.plot([l[j][0], l[j + 1][0]], [l[j][1], l[j + 1][1]], \
              c="r", linewidth=0.5)
      j += 1
    ax.scatter([e[0] for e in l], \
               [e[1] for e in l], s=12, c="b", marker=".")
    ax.set_title(name_algorithm)
    #ax.set_xlabel("distancia")
    #ax.set_ylabel("coste")
    # good data
    ax.set_xlim([1500, 20000])
    ax.set_ylim([400, 20000])
    # bad data
    #ax.set_xlim([0, 15000000])
    #ax.set_ylim([100000, 500000])
    #ax.locator_params(nbins=4)
    ax.grid(color="grey", linestyle="-", linewidth=0.2)
    ax.xaxis.set_major_locator(ticker.LinearLocator(numticks=2))
    #ax.autoscale()
    i += 1
  plt.tight_layout()
  #plt.suptitle("jMetal", fontsize=16)
  fig.text(0.5, 0.02, "distancia", ha="center", va="center")
  fig.text(0.02, 0.5, "coste", ha="center", va="center", rotation="vertical")
  plt.subplots_adjust(wspace=0.9, top=0.9)
  plt.savefig("{}/jMetal.png".format(os.getcwd()))
  plt.show()
