#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import matplotlib.pyplot as plt
import numpy as np
import os
from geopy.geocoders import Nominatim
from mpl_toolkits.basemap import Basemap


CITIES = {0: "Gerona, España", 1: "Barcelona, España", \
          2: "Tarragona, España", 3: "Tortosa, España", \
          4: "Castellon, España", 5: "Valencia, España", \
          6: "Alicante, España", 7: "Murcia, España", \
          8: "Almeria, España", 9: "Malaga, España", \
          10: "Marbella, España", 11: "Gibraltar, Gibraltar", \
          12: "Cadiz, España", 13: "Huelva, España", \
          14: "Badajoz, España", 15: "Caceres, España", \
          16: "Salamanca, España"}


def midpoint(xpoints, ypoints):
  return (xpoints[0] + xpoints[1]) / 2, (ypoints[0] + ypoints[1]) / 2




if __name__ == "__main__":
  with open("{}/sample".format(os.getcwd())) as f:
    for line in f:
      aux = line[:-1].split(" ")
      i = 0
      while i < len(aux):
        aux[i] = int(aux[i])
        i += 1
      m = Basemap(projection="merc", llcrnrlat=35.9, urcrnrlat=43.5, \
                  llcrnrlon=-8, urcrnrlon=4, resolution="i", \
                  area_thresh=500.)
      m.drawcoastlines(linewidth=0.5)
      m.drawcountries(linewidth=0.5)
      m.drawstates()
      m.fillcontinents(color="#04BAE3", lake_color="#FFFFFF")
      m.drawmapboundary(fill_color="#FFFFFF")

      geolocator = Nominatim()
      coor_x = []
      coor_y = []
      for num in aux:
        location = geolocator.geocode(CITIES[num])
        x, y = m(location.longitude, location.latitude)
        coor_x.append(x)
        coor_y.append(y)
        m.plot(x, y, "ro", alpha=0.6)

      m.plot(coor_x, coor_y, "g-", linewidth=0.3)
      
      """
      i = 0
      while i < len(coor_x) - 1:
        m.plot([coor_x[i], coor_x[i + 1]], [coor_y[i], coor_y[i + 1]], \
               "g-", linewidth=0.7)
        i += 1
      """

      """
      i = 0
      while i < len(coor_x) - 1:
        plt.annotate("", xy=(coor_x[i + 1], coor_y[i + 1]), \
                     xytext=(coor_x[i], coor_y[i]), \
                     arrowprops={"arrowstyle": "->"}, va="center")
        i += 1
      """
      i = 0
      while i < len(coor_x) - 1:
        """
        plt.arrow(coor_x[i], coor_y[i], coor_x[i + 1] - coor_x[i], \
                  coor_y[i + 1] - coor_y[i], fc="k", ec="k", \
                  head_width=27000, head_length=27000, \
                  length_includes_head=True, linewidth=0.1)
        """
        """
        plt.arrow(aux[0], aux[1], coor_x[i + 1] - aux[0], \
                  coor_y[i + 1] - aux[1], fc="k", ec="k", \
                  head_width=0, head_length=0, \
                  length_includes_head=True, linewidth=0.1)
        """
        aux = midpoint([coor_x[i], coor_x[i + 1]], \
                       [coor_y[i], coor_y[i + 1]])
        plt.arrow(coor_x[i], coor_y[i], aux[0] - coor_x[i], \
                  aux[1] - coor_y[i], fc="grey", ec="k", \
                  head_width=27000, head_length=27000, \
                  length_includes_head=True, linewidth=0, overhang=0.3)   # linewidth=0.1
        i += 1
      m.plot(coor_x[0], coor_y[0], "bo", label="inicio", alpha=0.6)
      #plt.scatter(coor_x[0], coor_y[0], marker="o", color="blue", \
      #            label="inicio")
      m.plot(coor_x[-1], coor_y[-1], "yo", label="fin", alpha=0.6)
      plt.title("SPAIN TOUR")
      plt.legend(numpoints=1, scatterpoints=1)
      plt.savefig("{}/route.png".format(os.getcwd()))
      plt.show()
