#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import array
# plot the results using matplotlib
import matplotlib.pyplot as plt
import numpy
import os
import random
import subprocess
from deap import algorithms
from deap import base
from deap import creator
from deap import tools



cost = []
distance = []
with open(os.getcwd() + "/cost.txt", "r") as f:
  for line in f.readlines():
    aux = []
    aux_1 = line.split(" ")
    for e in aux_1:
      aux.append(float(e))
    cost.append(tuple(aux))
with open(os.getcwd() + "/distance.txt", "r") as f:
  for line in f.readlines():
    aux = []
    aux_1 = line.split()
    for e in aux_1:
      aux.append(float(e))
    distance.append(tuple(aux))
#print(cost)
#print(distance)


def initIndividual(icls, content):
  return icls(content)



def initPopulation(pcls, ind_init):
  res = []
  for _ in range(100):
    aux = random.sample(list(range(len(cost))), len(cost))
    print(aux)
    res.append(array.array("i", aux))
  return pcls(ind_init(e) for e in res)


IND_SIZE = len(cost)
creator.create("FitnessMin", base.Fitness, weights=(-1.0, -1.0))
creator.create("Individual", array.array, typecode='i', \
               fitness=creator.FitnessMin)
toolbox = base.Toolbox()
# attribute generator
toolbox.register("indices", random.sample, range(IND_SIZE), IND_SIZE)
# structure initializers
#toolbox.register("individual", tools.initIterate, creator.Individual, toolbox.indices)
#toolbox.register("population", tools.initRepeat, list, toolbox.individual)
toolbox.register("population_guess", initPopulation, list, \
                 creator.Individual)



def evalTSP(individual):
  #print(individual)
  d = 0.0
  c = 0.0
  ini = individual[0]
  fin = individual[len(individual) - 1]
  #r = 0.0
  i = 0
  while i < len(individual) - 1:
    d += distance[individual[i]][individual[i + 1]]
    c += cost[individual[i]][individual[i + 1]]
    #r += (distance[individual[i]][individual[i + 1]] + cost[individual[i]][individual[i + 1]]) / 3
    i += 1
  d += distance[fin][ini]
  c += cost[fin][ini]
  return d, c


toolbox.register("mate", tools.cxPartialyMatched)
toolbox.register("mutate", tools.mutShuffleIndexes, indpb=0.05)
#toolbox.register("select", tools.selTournament, tournsize=3)
#toolbox.register("select", tools.selNSGA2)
toolbox.register("select", tools.selSPEA2)
toolbox.register("evaluate", evalTSP)

def main():
  random.seed(random.randint(0, 169))

  #pop = toolbox.population(n=5)
  pop = toolbox.population_guess()
  #print(pop)

  #hof = tools.HallOfFame(1)
  hof = tools.ParetoFront()
  stats = tools.Statistics(lambda ind: ind.fitness.values)
  stats.register("avg", numpy.mean)
  stats.register("std", numpy.std)
  stats.register("min", numpy.min)
  stats.register("max", numpy.max)

  i = 0
  l = [50, 100, 150, 200]
  fronts = {}
  # display the results
  fig = plt.figure()
  for iterations in l:
    fronts[iterations] = [] 
    a = algorithms.eaSimple(pop, toolbox, 0.7, 0.2, iterations, \
                            stats=stats, halloffame=hof)
    #print(len(a[0]))   # population length
    if iterations == 200:
      solutions_file = open("{}/deap/SPEA2_nondominated_solutions." \
                            "txt".format(os.getcwd()), "w")
      fitness_file = open("{}/deap/SPEA2_nondominated_solutions." \
                          "txt".format(os.getcwd()), "w")
    for e in hof:
      print([x for x in e], e.fitness.values)
      if iterations == 200:
        aux = ""
        for elem in e:
          aux += "{} ".format(elem)
        aux = aux[:-1]
        aux += "\n"
        solutions_file.write(aux)
        fitness_file.write("{} {}".format(e.fitness.values[0], \
                                          e.fitness.values[1]))
      fronts[iterations].append([x for x in e])
    
    ax = fig.add_subplot(2, 2, i+1)
    ax.scatter([s.fitness.values[0] for s in a[0]], \
               [s.fitness.values[1] for s in a[0]], c="g", marker=".")
    j = 0
    while j < len(hof) - 1:
      ax.plot([hof[j].fitness.values[0], hof[j + 1].fitness.values[0]], \
              [hof[j].fitness.values[1], hof[j + 1].fitness.values[1]], \
              c="r")
      j += 1
    ax.scatter([e.fitness.values[0] for e in hof], \
               [e.fitness.values[1] for e in hof], c="b", marker=".")
    ax.set_title("{} iteraciones".format(iterations))
    # good data
    ax.set_xlim([1500, 20000])
    ax.set_ylim([800, 20000])
    # bad data
    #ax.set_xlim([0, 15000000])
    #ax.set_ylim([100000, 500000])
    #ax.locator_params(nbins=4)
    #ax.autoscale()
    i += 1
  plt.tight_layout()
  #plt.suptitle("DEAP", fontsize=16)
  fig.text(0.5, 0.02, "distancia", ha="center", va="center")
  fig.text(0.02, 0.5, "coste", ha="center", va="center", rotation="vertical")
  plt.subplots_adjust(wspace=0.9, top=0.9)
  plt.savefig("{}/deap.png".format(os.getcwd()))
  plt.show()
  solutions_file.close()
  print(fronts)

  return pop, stats, hof




if __name__ == "__main__":
  subprocess.check_output("rm -rf {}/deap".format(os.getcwd()), shell=True)
  subprocess.check_output("mkdir {}/deap".format(os.getcwd()), shell=True)
  main()
