#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# plot the results using matplotlib
import matplotlib.pyplot as plt
# http://queirozf.com/entries/matplotlib-pyplot-by-example
import matplotlib.ticker as ticker
# https://matplotlib.org/examples/ticks_and_spines/tick-locators.html
import numpy
import os
import pickle
import subprocess
import time
from platypus import experiment, six, ProcessPoolEvaluator
# PESA2 not works in Python3, in Python2 yes
# population_size is necessary in MOEAD
from platypus.algorithms import EpsMOEA, EpsNSGAII, GDE3, IBEA, MOEAD, NSGAII, NSGAIII, PAES, PESA2, SPEA2
# Archive is a elitist set
from platypus.core import Archive, nondominated, Problem
# Variator = crossover or mutation or (crossover + mutation, use platypus.operator.GAOperator)
from platypus.operators import CompoundOperator, GAOperator, Mutation, PM, PMX, SBX, Variator
from platypus.types import Permutation
from platypus.weights import random_weights



MATRIX = None
start_time = time.time()



def multi_tsp(chromosome):
  global MATRIX
  distance = 0
  cost = 0
  ini = chromosome[0][0]
  fin = chromosome[0][len(chromosome[0]) - 1]
  #print(chromosome)
  #print(chromosome[0])
  i = 0
  while i < len(chromosome[0]) - 1:
    distance += MATRIX[chromosome[0][i]][chromosome[0][i + 1]][0]
    i += 1
  distance += MATRIX[fin][ini][0]
  i = 0
  while i < len(chromosome[0]) - 1:
    cost += MATRIX[chromosome[0][i]][chromosome[0][i + 1]][1]
    i += 1
  cost += MATRIX[fin][ini][1]
  return [distance, cost]



class MultiTSP(Problem):

  def __init__(self, matrix):
    super(MultiTSP, self).__init__(1, 2)
    self.matrix = matrix
    #super().__init__(1, 2)   # in Python3
    self.types[:] = Permutation(range(len(matrix)))
    #self.constraints[:] = "<=0"

  def evaluate(self, solution):
    #print(time.time() - start_time)
    #print(solution.variables[0])
    distance = 0
    cost = 0
    ini = solution.variables[0][0]
    fin = solution.variables[0][len(solution.variables[0]) - 1]
    i = 0
    while i < len(solution.variables[0]) - 1:
      distance += self.matrix[solution.variables[0][i]][solution.variables[0][i + 1]][0]
      i += 1
    distance += self.matrix[fin][ini][0]
    i = 0
    while i < len(solution.variables[0]) - 1:
      cost += self.matrix[solution.variables[0][i]][solution.variables[0][i + 1]][1]
      i += 1
    cost += self.matrix[fin][ini][1]
    solution.objectives[:] = [distance, cost]
    #x = solution.variables[:]
    #solution.constraints[:] = [-x + y - 1, x + y - 7]




if __name__ == "__main__":
  f = open("tsp_python.txt", "rb")
  MATRIX, CITIES = pickle.load(f)

  subprocess.check_output("rm -rf {}/platypus".format(os.getcwd()), \
                          shell=True)
  subprocess.check_output("mkdir {}/platypus".format(os.getcwd()), \
                          shell=True)

  """
  #problem = Problem(1, 2)
  ##problem = Problem(2, 2)   # in problem.function pass two chromosomes
  #problem.types[:] = Permutation(range(len(MATRIX)))
  ##problem.constraints[:] = "<=0"
  #problem.function = multi_tsp

  ##algorithm = NSGAII(problem, variator=None)
  algorithm = EpsMOEA(problem, epsilons=4)
  algorithm = EpsNSGAII(problem, epsilons=4)
  algorithm = GDE3(problem, population_size=100, variator=PMX())
  algorithm = MOEAD(problem, population_size=200, \
  weight_generator=random_weights)
  algorithm = NSGAII(problem)
  algorithm = NSGAIII(problem, divisions_outer=12)
  algorithm = PAES(problem)
  algorithm = PESA2(problem)
  # in SPEA2 population.size == archive.size, see self._truncate function
  algorithm = SPEA2(problem, population_size=200, k=20)
  algorithm.run(100)
  # display the results
  for solution in algorithm.result:
    print(solution.objectives)
  """

  problem = MultiTSP(MATRIX)
  algorithms = [(EpsMOEA, {"epsilons": 4}), \
                (EpsNSGAII, {"epsilons": 4}), \
                #GDE3, \
                IBEA, \
                (MOEAD, {"population_size": 100, \
                         "weight_generator": random_weights}), \
                NSGAII, \
                (NSGAIII, {"divisions_outer": 12}), \
                PAES, \
                PESA2, \
                SPEA2 \
                ]
  # run the experiment using Python 3's concurrent futures for parallel 
  # evaluation
  with ProcessPoolEvaluator() as evaluator:
    results = experiment(algorithms, problem, seeds=1, nfe=4000, \
                         evaluator=evaluator)   # nfe=100
  # display the results
  fig = plt.figure()
  fronts = {}
  for i, algorithm in enumerate(six.iterkeys(results)):
    fronts[algorithm] = []
    solutions_file = open("{}/platypus/{}_solutions." \
                          "txt".format(os.getcwd(), algorithm), "w")
    fitness_solutions_file = open("{}/platypus/{}_fitness." \
                                  "txt".format(os.getcwd(), algorithm), "w")
    nondominated_solutions_file = open("{}/platypus/{}_" \
                                       "nondominated_solutions." \
                                       "txt".format(os.getcwd(), \
                                                    algorithm), "w")
    fitness_nondominated_solutions_file = open("{}/platypus/{}_" \
                                               "nondominated_fitness." \
                                               "txt".format(os.getcwd(), \
                                                            algorithm), "w")
    result = results[algorithm]["MultiTSP"][0]
    ax = fig.add_subplot(2, 5, i+1)
    for solution in result:
      aux = ""
      for elem in solution.variables[0]:
        aux += "{} ".format(elem)
      aux = aux[:-1]
      aux += "\n"
      solutions_file.write(aux)
      fitness_solutions_file.write(str(solution.objectives[0]) + " " + \
                                   str(solution.objectives[1]) + "\n")
    ax.scatter([s.objectives[0] for s in result], \
               [s.objectives[1] for s in result], c="g", marker=".")
    nondominated_solutions = nondominated(result)
    print(algorithm)
    for solution in nondominated(result):
      print(solution)
      aux = ""
      for elem in solution.variables[0]:
        aux += "{} ".format(elem)
      aux = aux[:-1]
      aux += "\n"
      nondominated_solutions_file.write(aux)
      fitness_nondominated_solutions_file.write(
                     str(solution.objectives[0]) + " " + \
                     str(solution.objectives[1]) + "\n")
      fronts[algorithm].append([solution.objectives[0], \
                                solution.objectives[1]])
    l = list((s.objectives[0], s.objectives[1]) \
             for s in nondominated_solutions)
    l = sorted(l, key=lambda x : (x[0], x[1]))
    i = 0
    while i < len(l) - 1:
      ax.plot([l[i][0], l[i + 1][0]], [l[i][1], l[i + 1][1]], c="r")
      i += 1
    ax.scatter([e[0] for e in l], \
               [e[1] for e in l], c="b", marker=".")
    ax.set_title(algorithm)
    # good data
    ax.set_xlim([1500, 10000])
    ax.set_ylim([800, 6000])
    #ax.locator_params(nbins=4)
    # bad data
    #ax.set_xlim([0, 15000000])
    #ax.set_ylim([100000, 500000])
    #ax.set_xlabel("distancia")
    #ax.set_ylabel("coste")
    #ax.autoscale()
    ax.yaxis.set_major_locator(plt.MaxNLocator(6))
    solutions_file.close()
    nondominated_solutions_file.close()
    #plt.xticks(numpy.arange(1000, 15000, step=6000))
    #plt.xticks(numpy.arange(0, 15000000, step=14999999))
    ax.xaxis.set_major_locator(ticker.LinearLocator(numticks=2))
  #plt.yticks()
  plt.tight_layout()
  #plt.suptitle("Platypus", fontsize=16)
  fig.text(0.5, 0.02, "distancia", ha="center", va="center")
  fig.text(0.02, 0.5, "coste", ha="center", va="center", rotation="vertical")
  ax = fig.add_subplot(2, 5, 10)
  ax.set_title("Frentes")
  for algorithm in fronts:
    l = sorted(fronts[algorithm], key=lambda x : (x[0], x[1]))
    i = 0
    while i < len(l) - 1:
      ax.plot([l[i][0], l[i + 1][0]], [l[i][1], l[i + 1][1]], c="r")
      i += 1
    ax.scatter([e[0] for e in l], \
               [e[1] for e in l], marker=".", label=algorithm)
  ax.xaxis.set_major_locator(ticker.LinearLocator(numticks=2))
  plt.subplots_adjust(wspace=0.9, top=0.9)
  plt.savefig("{}/platypus.png".format(os.getcwd()), dpi=800)
  plt.show()

  
  """
  algorithm = EpsNSGAII(problem, epsilons=4)
  #print(algorithm.variator)
  algorithm.run(3500)

  # do this if algorithm.run < 200
  #print(len(algorithm.result))
  feasible_solutions = [s for s in algorithm.result if s.feasible]
  #print(feasible_solutions)
  #print(len(feasible_solutions))
  # algorithm.result == feasible_solutions == True in this problem!!!
  #print(algorithm.result == feasible_solutions)
  nondominated_solutions = nondominated(algorithm.result)
  for solution in nondominated(algorithm.result):
    print(solution)
  #print(len(nondominated_solutions))
  plt.plot([s.objectives[0] for s in algorithm.result],
           [s.objectives[1] for s in algorithm.result], "g.")
  #plt.plot([s.objectives[0] for s in feasible_solutions],
  #         [s.objectives[1] for s in feasible_solutions], "go")
  l = list((s.objectives[0], s.objectives[1]) \
      for s in nondominated_solutions)
  l = sorted(l, key=lambda x : (x[0], x[1]))
  plt.plot([e[0] for e in l], [e[1] for e in l], "r-")
  plt.plot([e[0] for e in l], [e[1] for e in l], "b.")
  # google
  plt.xlim([0, 15000000])
  plt.ylim([120000, 500000])
  plt.xticks(numpy.arange(0, 15000000, step=14999999))
  plt.xlabel("distancia")
  plt.ylabel("coste")
  plt.show()
  """
