#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# https://stackoverflow.com/questions/39332053/using-scipy-interpolate-interpn-to-interpolate-a-n-dimensional-array


import matplotlib.pyplot as plt
import numpy as np
from matplotlib import cm
from mpl_toolkits.mplot3d import Axes3D
from scipy.interpolate import interpn




if __name__ == "__main__":
  # set up grid and array of values
  x1 = np.arange(10)
  x2 = np.arange(10)
  arr = x1 + x2[:, np.newaxis]

  # set up grid for plotting
  X, Y = np.meshgrid(x1, x2)

  # plot the values as a surface plot to depict
  fig = plt.figure()
  ax = fig.gca(projection="3d")
  surf = ax.plot_surface(X, Y, arr, rstride=1, cstride=1, cmap=cm.jet, \
                         linewidth=0, alpha=0.8)
  fig.colorbar(surf, shrink=0.5, aspect=5)
  

  interp_x = 3.5           # only one value on the x1-axis
  interp_y = np.arange(10) # a range of values on the x2-axis

  # note the following two lines that are used to set up the
  # interpolation points as a 10x2 array!
  interp_mesh = np.array(np.meshgrid(interp_x, interp_y))
  interp_points = np.rollaxis(interp_mesh, 0, 3).reshape((10, 2))

  # perform the interpolation
  interp_arr = interpn((x1, x2), arr, interp_points)

  # plot the result
  ax.scatter(interp_x * np.ones(interp_y.shape), interp_y, interp_arr, \
             s=20, c="k", depthshade=False)
  plt.xlabel("x1")
  plt.ylabel("x2")

  plt.show()
